# coding: utf-8
from abc import ABCMeta
from abc import abstractmethod
import logging
import signal
import sys

from mvdcommon.consumer import consume
from mvdcommon.sync.services.base import BaseSyncService


class BaseRabbitMQClient(metaclass=ABCMeta):
    """Базовый RabbitMQ клиент для синхронизации данных."""

    def __init__(self, host: str, queue: str, user: str, password: str):
        self._host = host
        self._queue = queue
        self._user = user
        self._password = password

    @property
    @abstractmethod
    def sync_service(self) -> BaseSyncService:
        """Возвращает сервис для обработки данных из сообщения."""
        raise NotImplementedError

    def _process_data(self, data: str) -> None:
        """Передает на обработку данные из сообщения"""
        return self.sync_service.sync(data)

    def _callback(self, ch, method, properties, data):
        """Обрабатывает полученное сообщение."""
        logging.getLogger('sync').info(
            'Для очереди %s получено сообщение',
            self._queue
        )
        try:
            self._process_data(data)
        except Exception as exc:  # noqa: W0703
            logging.getLogger('sync').error(str(exc))

        ch.basic_ack(delivery_tag=method.delivery_tag)

    def consume(self):
        """Ождает сообщения в очереди."""
        signal.signal(
            signal.SIGTERM,
            lambda sn, sf: sys.exit(0)
        )
        logging.getLogger('sync').info(
            'Запущен rabbitMQ клиент для %s@%s/%s',
            self._user,
            self._host,
            self._queue
        )
        consume(
            host=self._host,
            queue=self._queue,
            user=self._user,
            password=self._password,
            callback=self._callback
        )
